@@ -8,7 +8,35 @@ use crate::options::*;
 use crate::util::OneWayOwned;
 use crate::writer::Writer;
 use std::borrow::Borrow;
-use std::io::Write;
+
+enum View
+{
+	PitchYaw(f64, f64),
+	Map,
+}
+
+impl View
+{
+	fn write_out(&self, writer: &mut dyn Writer)
+	{
+		match self
+		{
+			Self::PitchYaw(pitch, yaw) =>
+			{
+				writeln!(writer, "set view {:.12e},{:.12e}", pitch, yaw);
+			}
+			Self::Map =>
+			{
+				writer.write_str("set view map\n");
+			}
+		}
+	}
+
+	fn reset_state(&self, writer: &mut dyn Writer)
+	{
+		writer.write_str("unset view\n");
+	}
+}
 
 /// 3D axes that is used for drawing 3D plots
 pub struct Axes3D
@@ -21,6 +49,7 @@ pub struct Axes3D
 	contour_levels: Option<Vec<f64>>,
 	contour_style: ContourStyle,
 	contour_label: AutoOption<String>,
+	view: Option<View>,
 }
 
 impl Axes3D
@@ -36,6 +65,7 @@ impl Axes3D
 			contour_levels: None,
 			contour_style: Linear,
 			contour_label: Auto,
+			view: None,
 		}
 	}
 
@@ -166,18 +196,14 @@ impl Axes3D
 	/// * `yaw` - Yaw, in degrees. Value of 0 is looking at the XZ plane, Y point into the screen.
 	pub fn set_view<'l>(&'l mut self, pitch: f64, yaw: f64) -> &'l mut Self
 	{
-		writeln!(
-			&mut self.common.commands,
-			"set view {:.12e},{:.12e}",
-			pitch, yaw
-		);
+		self.view = Some(View::PitchYaw(pitch, yaw));
 		self
 	}
 
 	/// Sets the view to be a map. Useful for images and contour plots.
 	pub fn set_view_map<'l>(&'l mut self) -> &'l mut Self
 	{
-		writeln!(&mut self.common.commands, "set view map");
+		self.view = Some(View::Map);
 		self
 	}
 
@@ -194,8 +220,9 @@ impl Axes3D
 	pub fn set_z_label<'l>(&'l mut self, text: &str, options: &[LabelOption<&str>])
 		-> &'l mut Self
 	{
-		self.get_common_data_mut()
-			.set_label_common(ZLabel, text, options);
+		self.z_axis
+			.label
+			.set(text.into(), options.to_one_way_owned());
 		self
 	}
 
@@ -363,31 +390,14 @@ impl Axes3D
 		self.contour_label = label.map(|l| l.to_string());
 		self
 	}
-}
 
-impl AxesCommonPrivate for Axes3D
-{
-	fn get_common_data_mut(&mut self) -> &mut AxesCommonData
+	pub(crate) fn reset_state(&self, writer: &mut dyn Writer)
 	{
-		&mut self.common
+		self.common.reset_state(writer);
+		self.view.as_ref().map(|v| v.reset_state(writer));
 	}
 
-	fn get_common_data(&self) -> &AxesCommonData
-	{
-		&self.common
-	}
-}
-
-impl AxesCommon for Axes3D {}
-
-pub(crate) trait Axes3DPrivate
-{
-	fn write_out(&self, writer: &mut Writer, version: GnuplotVersion);
-}
-
-impl Axes3DPrivate for Axes3D
-{
-	fn write_out(&self, w: &mut Writer, version: GnuplotVersion)
+	pub(crate) fn write_out(&self, w: &mut dyn Writer, auto_layout: bool, version: GnuplotVersion)
 	{
 		fn clamp<T: PartialOrd>(val: T, min: T, max: T) -> T
 		{
@@ -405,11 +415,6 @@ impl Axes3DPrivate for Axes3D
 			}
 		}
 
-		if self.common.elems.len() == 0
-		{
-			return;
-		}
-
 		if self.contour_base || self.contour_surface
 		{
 			write!(w, "set contour ");
@@ -442,7 +447,7 @@ impl Axes3DPrivate for Axes3D
 				}
 			};
 
-			fn set_cntrparam<F: FnOnce(&mut Writer)>(w: &mut Writer, wr: F)
+			fn set_cntrparam<F: FnOnce(&mut dyn Writer)>(w: &mut dyn Writer, wr: F)
 			{
 				write!(w, "set cntrparam ");
 				wr(w);
@@ -517,7 +522,7 @@ impl Axes3DPrivate for Axes3D
 			});
 		}
 
-		self.common.write_out_commands(w, version);
+		self.common.write_out_commands(w, auto_layout, version);
 		self.z_axis.write_out_commands(w, version);
 		let mut grid_axes = vec![];
 		if self.common.x_axis.grid
@@ -536,7 +541,23 @@ impl Axes3DPrivate for Axes3D
 		{
 			grid_axes.push(self.z_axis.axis);
 		}
+		self.view.as_ref().map(|v| v.write_out(w));
 		self.common.write_grid_options(w, &grid_axes, version);
 		self.common.write_out_elements("splot", w, version);
 	}
 }
+
+impl AxesCommonPrivate for Axes3D
+{
+	fn get_common_data_mut(&mut self) -> &mut AxesCommonData
+	{
+		&mut self.common
+	}
+
+	fn get_common_data(&self) -> &AxesCommonData
+	{
+		&self.common
+	}
+}
+
+impl AxesCommon for Axes3D {}
